/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.communication.topology;

import java.util.Set;

import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.kernel.configuration.SubDomainConfiguration;
import org.ow2.petals.kernel.configuration.ContainerConfiguration.ContainerState;
import org.ow2.petals.kernel.configuration.DomainConfiguration;

/**
 * 
 * @author Roland NAUDIN - eBMWebSourcing
 * @author chamerling
 * 
 * @since Petals 2.0
 */
public interface TopologyService {

    /**
     * Get the domain configuration.
     *
     * @return the domain configuration
     * @throws PetalsException
     *             Failed to retrieve the domain configuration
     */
    DomainConfiguration getDomainConfiguration() throws PetalsException;

    /**
     * Add a subdomain to the local topology.
     *
     * @param addedSubdomain
     * @throws PetalsException
     */
    void addSubdomainConfiguration(SubDomainConfiguration addedSubdomain) throws PetalsException;

    /**
     * Remove subdomain configuration.
     *
     * @param removedSubdomain
     * @throws PetalsException
     */
    void removeSubdomainConfiguration(String removedSubdomainName) throws PetalsException;

    /**
     * Add the given container to the topology
     *
     * @param addedContainer the container to add to the current topology
     * @throws PetalsException
     *              Failed to add the container to the current topology
     */
    void addContainerConfiguration(ContainerConfiguration addedContainer) throws PetalsException;

    /**
     * Remove container configuration.
     *
     * @param removedContainer
     * @throws PetalsException
     */
    void removeContainerConfiguration(ContainerConfiguration removedContainer) throws PetalsException;

    /**
     * Get all the sub-domain configurations.
     * 
     * @return a set of {@link ContainerConfiguration}
     * @throws PetalsException
     *             Failed to retrieve the containers configuration
     */
    Set<SubDomainConfiguration> getSubDomainsConfiguration() throws PetalsException;

    /**
     * Get the container configuration of the given container name.
     * 
     * @param containerName
     *            The name of the targeted container
     * @return The container configuration
     * @throws PetalsException
     *             Failed to retrieve the container configuration
     */
    ContainerConfiguration getContainerConfiguration(String containerName) throws PetalsException;

    /**
     * Set the given state to the give container
     * 
     * @param containerName
     * @param state
     *            The {@code ontainerConfiguration.ContainerState}
     * @throws PetalsException
     *             Failed to set the state to the container
     */
    void setContainerState(String containerName, ContainerConfiguration.ContainerState state)
            throws PetalsException;

    /**
     * Test if the container is started
     * 
     * @param containerName
     * @return
     * @throws PetalsException
     */
    boolean isContainerStarted(String containerName) throws PetalsException;

    /**
     * Get all the containers where the state is the same than the given one. If
     * the state is null, get all the containers...
     * 
     * @param object
     * @return
     */
    Set<ContainerConfiguration> getContainersConfiguration(ContainerState state)
            throws PetalsException;

    /**
     * Validate the dynamic topology
     *
     * @throws ConfigurationException
     */
    void validateDynamicTopology() throws PetalsException;

    void registerLocalContainerOnMaster() throws PetalsException;

    void updateTopology() throws PetalsException;

    Set<ContainerConfiguration> getContainersConfigurationsForLocalSubdomain();

    public boolean hasValidLocalContainerDynamicTopologyConfiguration();
}
