/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: CreateAndRegisterSULifeCycleTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.net.URI;
import java.net.URL;

import javax.jbi.JBIException;

import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.Task;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;

/**
 * This task creates a service unit lifecycle and registers this lifecycle into
 * the sa lifecycle <br/>
 * Inputs : {@link Jbi} of the SU, {@link ServiceAssemblyLifeCycle}, SU install
 * root {@link URI} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class CreateAndRegisterSULifeCycleTask implements Task {
    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminService adminService;

    public CreateAndRegisterSULifeCycleTask(AdminService adminService) {
        this.adminService = adminService;
    }

    public void execute(Context context) throws Exception {
        URL installUrl = context.getSuInstallUrl();
        ServiceUnit serviceUnitInSa = context.getSuDescriptorInSa();
        ServiceAssemblyLifeCycle saLifeCycle = context.getSaLifeCycle();
        // Create SU lifecycle
        ServiceUnitLifeCycle suLifeCycle = new ServiceUnitLifeCycle(serviceUnitInSa, installUrl
                .getPath(), this.adminService);
        // Register this SU lifecycle into the SA lifecycle
        this.registerSULifeCycle(suLifeCycle, saLifeCycle);
    }

    public void undo(Context context) throws Exception {
        if ((context.getSuDescriptorInSa() != null) && (context.getSaLifeCycle() != null)) {
            this.unregisterSULifeCycle(context.getSaLifeCycle(), context.getSuDescriptorInSa());
        }
    }

    /**
     * Register the {@link ServiceUnitLifeCycle} to the
     * {@link ServiceAssemblyLifeCycle}.
     * 
     * @param suLifeCycle
     *            the {@link ServiceUnitLifeCycle} to register
     * @param saLifeCycle
     *            the {@link ServiceAssemblyLifeCycle} where
     *            {@link ServiceUnitLifeCycle} will be registered
     * @param serviceUnit
     *            the service unit production element
     * @throws ManagementException
     *             if it fails to register {@link ServiceUnitLifeCycle} to
     *             {@link ServiceAssemblyLifeCycle}
     */
    protected void registerSULifeCycle(ServiceUnitLifeCycle suLifeCycle,
            ServiceAssemblyLifeCycle saLifeCycle) throws ManagementException {
        try {
            saLifeCycle.registerSU(suLifeCycle);
        } catch (JBIException e) {
            throw new ManagementException("Error while registering the service unit '"
                    + suLifeCycle.getSuName()
                    + "' life cycle into the service assembly life cycle '"
                    + saLifeCycle.getServiceAssembly().getIdentification().getName() + "'", e);
        }
    }

    /**
     * Unregister the {@link ServiceUnitLifeCycle} from the
     * {@link ServiceAssemblyLifeCycle}.
     * 
     * @param saLifeCycle
     *            the {@link ServiceAssemblyLifeCycle} where
     *            {@link ServiceUnitLifeCycle} will be unregistered
     * @param serviceUnit
     *            the service unit production element of the su to unregister
     */
    private void unregisterSULifeCycle(ServiceAssemblyLifeCycle saLifeCycle, ServiceUnit serviceUnit) {
        for (ServiceUnitLifeCycle suLifeCycle : saLifeCycle.getServiceUnitLifeCycles()) {
            if (suLifeCycle.getSuName().equals(DeploymentUtils.getServiceUnitName(serviceUnit))) {
                saLifeCycle.unregisterSU(suLifeCycle);
                break;
            }
        }

    }

}
