/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: CreateXMLDeploymentResultTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.deploy;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.Task;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.jbi.management.util.XMLResult;
import org.ow2.petals.jbi.management.util.XMLResult.CauseFramework;
import org.ow2.petals.jbi.management.util.XMLResult.MessageType;
import org.ow2.petals.jbi.management.util.XMLResult.TaskResult;

/**
 * This task create framework task result. As we decide to create xml task
 * result only when deployment of the SA success, xml result only handles xml
 * component messages. All exceptions (caused by the deployment framework or
 * thrown by a component during the deployment of an su) are thrown and they
 * will stop the deployment process. It's a small violation of the
 * specification. <br/>
 * Inputs : {@link Jbi} <br/>
 * Outputs : * {@link XMLResult} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class CreateXMLDeploymentResultTask implements Task {

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.Map)
     */
    public void execute(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();
        XMLResult xmlResult = new XMLResult("deploy");
        this.createFrameworkTaskResult(DeploymentUtils.getServiceAssemblyName(descriptor), xmlResult);
        context.setXmlResult(xmlResult);
    }

    /**
     * Create the framework task result of the xml result message. Framework
     * task result is always a success because we decide that in other cases, an
     * exception is thrown.
     * 
     * @param saName
     *            the service assembly name
     * @param result
     *            framework task result
     */
    protected void createFrameworkTaskResult(String saName, XMLResult result) {
        result.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
                "Successfully deployed service assembly : {1}", new String[] { saName },
                CauseFramework.YES);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.Map)
     */
    public void undo(Context context) throws Exception {
        context.setXmlResult(null);
    }

}
