/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: RegisterSALifeCycleTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.net.URI;

import javax.jbi.JBIException;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.ow2.petals.container.ContainerService;
import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.server.FractalHelper;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task registers ServiceAssemblyLifeCycle into manager service map <br/>
 * Inputs : {@link Jbi}, {@link ServiceAssemblyLifeCycle} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class RegisterSALifeCycleTask extends AbstractLoggableTask {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminService adminService;

    protected ContainerService containerService;

    public RegisterSALifeCycleTask(LoggingUtil log, AdminService adminService,
            ContainerService containerService) {
        super(log);
        this.adminService = adminService;
        this.containerService = containerService;
    }

    public void execute(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();
        ServiceAssemblyLifeCycle saLifeCycle = this.createSALifeCycle(descriptor);
        this.adminService.registerServiceAssembly(saLifeCycle, DeploymentUtils
                .getServiceAssemblyName(descriptor));
        context.setSaLifeCycle(saLifeCycle);
    }

    /**
     * Creates a new {@link ServiceAssemblyLifeCycle}
     * 
     * @param installationRoot
     *            the root {@link URI} of the service assembly into the
     *            repository
     * @param descriptor
     *            the service assembly jbi descriptor
     * @return a new {@link ServiceAssemblyLifeCycle}
     * @throws PetalsException
     * @throws ADLException
     * @throws IllegalLifeCycleException
     * @throws NoSuchInterfaceException
     * @throws JBIException
     */
    protected ServiceAssemblyLifeCycle createSALifeCycle(Jbi descriptor) throws PetalsException,
            NoSuchInterfaceException, IllegalLifeCycleException, ADLException, JBIException {
        Component serviceAssemblyLifeCycleComponent = this.containerService
                .createServiceAssemblyLifeCycle(descriptor.getServiceAssembly());
        FractalHelper.startComponent(serviceAssemblyLifeCycleComponent);
        ServiceAssemblyLifeCycle serviceAssemblyLifeCycle = (ServiceAssemblyLifeCycle) serviceAssemblyLifeCycleComponent
                .getFcInterface("/content");
        return serviceAssemblyLifeCycle;

    }

    public void undo(Context context) throws Exception {
        if (context.getDescriptor() != null) {
            Jbi descriptor = context.getDescriptor();
            try {
                this.adminService.unregisterServiceAssembly(DeploymentUtils
                        .getServiceAssemblyName(descriptor));
            } catch (ManagementException e) {
                this.log.error("Failed to revert a RegisterSALifeCycleTask: " + e.getMessage());
            }
        }
        if (context.getSaLifeCycle() != null) {
            try {
                this.containerService.removeServiceAssemblyLifeCycle(context.getSaLifeCycle());
            } catch (PetalsException e) {
                this.log.error("Failed to revert a RegisterSALifeCycleTask: " + e.getMessage());
            }
        }
        context.setSaLifeCycle(null);
    }

}
