/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: RetrieveSAAndSUInstallRootsTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.Task;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.system.repository.RepositoryService;

/**
 * This task retrieves the SA install root and all SUs install roots <br/>
 * Inputs : {@link Jbi} <br/>
 * Outputs : SA install root {@link URI}, SU install roots {@link Map} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class RetrieveSAAndSUInstallRootsTask implements Task {

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositoryService;

    public RetrieveSAAndSUInstallRootsTask(RepositoryService repositoryService) {
        super();
        this.repositoryService = repositoryService;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.management.Task#execute(org.ow2.petals.jbi.management.Context)
     */
    public void execute(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();
        URL saInstallUrl = this.repositoryService.getServiceAssemblyInstallDirectory(
                DeploymentUtils.getServiceAssemblyName(descriptor)).toURL();
        Map<String, URL> suInstallUrls = this.retrieveSUInstallUrls(saInstallUrl, descriptor);
        context.setInstallUrl(saInstallUrl);
        context.setSuInstallUrls(suInstallUrls);
    }

    /**
     * Retrieve all SU installation root {@link URI}
     * 
     * @param descriptor
     *            the service assembly descriptor
     * @param installationRoot
     *            the SA installation root {@link URI}
     * @return the service unit install roots {@link Map}
     * @throws MalformedURLException
     * @throws PetalsException
     */
    protected Map<String, URL> retrieveSUInstallUrls(URL saInstallUrl, Jbi descriptor)
            throws MalformedURLException {
        Map<String, URL> suInstallUrls = new HashMap<String, URL>();
        List<ServiceUnit> sus = descriptor.getServiceAssembly().getServiceUnit();
        for (ServiceUnit unit : sus) {
            String suName = unit.getIdentification().getName();
            suInstallUrls.put(suName, new File(saInstallUrl.getPath(), suName).toURI().toURL());
        }
        return suInstallUrls;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.management.Task#undo(org.ow2.petals.jbi.management.Context)
     */
    public void undo(Context context) throws Exception {
        //nothing to do
    }

}
