/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.jbi.messaging.registry;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.Cardinality;
import org.objectweb.fractal.fraclet.annotation.annotations.type.Contingency;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.registry.api.config.Configuration;
import org.ow2.petals.registry.api.config.ConfigurationLoader;
import org.ow2.petals.registry.api.config.ConfigurationLoaderFactory;
import org.ow2.petals.registry.api.exception.LifeCycleException;
import org.ow2.petals.registry.api.exception.RegistryException;
import org.ow2.petals.registry.client.RegistryClientFactory;
import org.ow2.petals.registry.core.factory.RegistryFactory;
import org.ow2.petals.util.LoggingUtil;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.messaging.registry.EndpointRegistry.class))
public class StandaloneEndpointRegistryImpl extends BaseEndpointRegistry {
    /**
     * Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class)
    protected ConfigurationService configurationService;

    @Requires(name = LISTENER_FRACTAL_PREFIX, cardinality = Cardinality.COLLECTION, contingency = Contingency.OPTIONAL, signature = RegistryListener.class)
    protected Hashtable<String, Object> listeners = new Hashtable<String, Object>();

    /**
     * Monolog Logger instance.
     */
    @Monolog(name = "logger")
    protected Logger logger;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws Exception {
        this.log = new LoggingUtil(this.logger);
        URL regConfigURL = this.getClass().getResource(CONFIG);
        if (regConfigURL == null) {
            throw new IOException(
                    "Registry configuration file has not been found, check classpath for " + CONFIG);
        }

        File f = new File(regConfigURL.toURI());
        try {
            // FIXME : To be moved in the registry configuration part
            // load the registry configuration file
            ConfigurationLoader loader = ConfigurationLoaderFactory.getLoader(f);
            if (loader == null) {
                throw new RegistryException("Can not get a configuration loader for the file "
                        + f.getName());
            }

            Configuration config = null;
            try {
                config = loader.loadLocal(new FileInputStream(f));
            } catch (FileNotFoundException e) {
                throw new RegistryException(e);
            }

            // update things...
            final ContainerConfiguration localContainerConfiguration = this.configurationService
                    .getContainerConfiguration();
            config.setName("Registry@PEtALSESB-" + localContainerConfiguration.getName());
            config.setRootPath(localContainerConfiguration.getWorkDirectoryPath());

            this.registry = RegistryFactory.getInstance().loadLocal(config);

            if (this.registry == null) {
                throw new PetalsException("Registry can not be loaded!");
            }

            this.registry.init();
            this.registry.start();

        } catch (RegistryException e) {
            this.log.error(e.getMessage(), e);
            if ((this.registry != null)
                    && (this.registry.isInitialized() || this.registry.isStarted())) {
                try {
                    this.registry.stop();
                } catch (LifeCycleException e1) {
                    throw new org.ow2.petals.jbi.messaging.registry.RegistryException(e1);
                }
            }
            throw new org.ow2.petals.jbi.messaging.registry.RegistryException(e);
        }

        this.client = RegistryClientFactory.getNewClient(this.registry);
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Stopping...");
        }

        // delete remaining data...
        // delete all the local entries
        try {
            this.registry.delete(this.getRootPath(), true);
        } catch (RegistryException e) {
            if (this.log.isErrorEnabled()) {
                this.log.error(e.getMessage());
            }
        }

        try {
            this.registry.stop();
        } catch (LifeCycleException e) {
            throw new org.ow2.petals.jbi.messaging.registry.RegistryException(e);
        }
    }

    @Override
    protected final String getRootPath() {
        return "/endpoints/" + this.configurationService.getContainerConfiguration().getName()
                + "/";
    }

    /**
     * {@inheritDoc}
     */
    public List<RegistryListener> getListeners() {
        List<RegistryListener> result = null;
        if (this.listeners != null) {
            result = new ArrayList<RegistryListener>();
            for (Object o : this.listeners.values()) {
                if (o instanceof RegistryListener) {
                    result.add((RegistryListener) o);
                }
            }
        }
        return result;
    }
}
