/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.server;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;

/**
 * This class helps fractal management for PEtALS.
 * 
 * @author alouis, rnaudin
 * 
 */
public class FractalHelper {

    /**
     * Fractal Petals names.
     */
    public static final String PETALS_COMPOSITE = "Petals";

    /**
     * The Communication component name.
     */
    public static final String COMMUNICATION_COMPOSITE = "Communication";

    /**
     * The configuration service component name.
     */
    public static final String CONFIGURATION_COMPONENT = "ConfigurationServiceImpl";

    /**
     * The network service component name.
     */
    public static final String NETWORK_COMPONENT = "NetworkServiceImpl";

    /**
     * The topology service component name.
     */
    public static final String TOPOLOGY_COMPONENT = "TopologyServiceImpl";

    /**
     * The JNDI agent service component name.
     */
    public static final String JNDIAGENT_COMPONENT = "JNDIAgentServiceImpl";

    /**
     * The Jndi service component name.
     */
    public static final String JNDI_COMPONENT = "JNDIServiceImpl";

    /**
     * The JMX service component name.
     */
    public static final String JMX_COMPONENT = "JMXServiceImpl";

    /**
     * The JBI Management component name.
     */
    public static final String JBI_MANAGEMENT_COMPOSITE = "JBI-Management";

    /**
     * The Admin Service component name.
     */
    public static final String ADMIN_COMPONENT = "AdminServiceImpl";

    /**
     * The deployment component name.
     */
    public static final String DEPLOYMENT_COMPONENT = "DeploymentServiceImpl";

    /**
     * The installation component name.
     */
    public static final String INSTALLATION_COMPONENT = "InstallationServiceImpl";

    /**
     * The endpoint service component name.
     */
    public static final String ENDPOINT_COMPONENT = "EndpointRegistryImpl";

    /**
     * The system recovery component name.
     */
    public static final String SYSTEMRECOVERY_COMPONENT = "SystemRecoveryServiceImpl";

    /**
     * The autoloader component name.
     */
    public static final String AUTOLOADER_COMPONENT = "AutoLoaderServiceImpl";

    /**
     * The Router component name.
     */
    public static final String ROUTER_COMPONENT = "RouterServiceImpl";

    /**
     * The Petals Router Monitor component name.
     */
    public static final String ROUTER_MONITOR_COMPONENT = "RouterMonitorImpl";

    /**
     * the JBI messaging composite name.
     */
    public static final String JBI_MESSAGING_COMPOSITE = "JBI-Messaging";

    public static final String TRANSPORTER_COMPOSITE = "Transporter";

    public static final String STANDALONE_TRANSPORTER_COMPONENT = "StandaloneTransporter";

    public static final String LOCAL_TRANSPORTER_COMPONENT = "LocalTransporter";

    public static final String TCP_TRANSPORTER_COMPONENT = "TcpTransporter";

    /**
     * The Petals Transport Monitor component name.
     */
    public static final String TRANSPORTMONITORING_COMPONENT = "TransportMonitoring";

    /**
     * the transporters component name.
     */
    public static final String MEMORYTRANSPORTER_COMPONENT = "MemoryTransporter";

    /**
     * The Petals admin service component name.
     */
    public static final String PETALSADMIN_COMPONENT = "PetalsAdminServiceImpl";

    public static final String WEBSERVICEMANAGER_COMPONENT = "WebServiceManagerImpl";

    /**
     * The Container component name.
     */
    public static final String CONTAINER_COMPOSITE = "Container";

    /**
     * the FCAPPL binding name.
     */
    public static final String FCAPPL_BINDING = "fcAppl";

    /**
     * the context.
     */
    private static Map<Object, Object> context = null;

    /**
     * The exception raised during the Fractal factory creation.
     */
    private static ADLException factoryCreationADLException = null;

    /**
     * the fractal factory.
     */
    private static Factory factory = null;

    /**
     * Initialize the Fractal Factory at the loading of the Class
     */
    static {
        // check required fractal properties
        System.setProperty("fractal.provider", "org.objectweb.fractal.julia.Julia");
        System.setProperty("julia.loader", "org.objectweb.fractal.julia.loader.DynamicLoader");
        System.setProperty("julia.config", "julia.cfg");

        // Create Fractal ADL factory
        context = new HashMap<Object, Object>();

        try {
            factory = FactoryFactory.getFactory(FactoryFactory.FRACTAL_BACKEND, context);
        } catch (ADLException e) {
            factoryCreationADLException = e;
        }

    }

    /**
     * Get the factory to manage components.
     * 
     * @return The factory
     * @throws ADLException
     *             : occurs when the framework of Petals is incorrect
     */
    public static final Factory getFactory() throws ADLException {
        if (factoryCreationADLException != null) {
            throw factoryCreationADLException;
        }
        return factory;
    }

    /**
     * Create a new component.
     * 
     * @param name
     *            The class name of the component
     * @return the new component
     * @throws ADLException
     *             : occurs when the framework of Petals is incorrect
     */
    public static final synchronized Component createNewComponent(final String name)
            throws ADLException {
        return (Component) getFactory().newComponent(name, context);
    }

    /**
     * Create a new component.
     * 
     * @param name
     *            The class name of the component
     * @param context
     *            The map of parameters for the component
     * @return the new component
     * @throws ADLException
     *             : occurs when the framework of Petals is incorrect
     */
    public static final synchronized Component createNewComponent(final String name,
            Map<Object, Object> context) throws ADLException {
        return (Component) getFactory().newComponent(name, context);
    }

    /**
     * A utility function allowing to get a component from any content
     * controller.
     * 
     * @param parentContentController
     *            parentContentController
     * @param name
     *            component name
     * @return the component, null if not found
     */
    public static final Component getRecursiveComponentByName(
            final ContentController parentContentController, final String name) {

        // the component to be returned
        Component matchedComponent = null;

        // the subcomponent content controller
        ContentController subContentController = null;

        // List content controller subcomponents
        for (Component component : parentContentController.getFcSubComponents()) {

            // if the component is a composite, search the matching component
            // recursively
            try {
                subContentController = Fractal.getContentController(component);
                if (subContentController.getFcSubComponents().length > 0) {
                    matchedComponent = getRecursiveComponentByName(subContentController, name);
                    if (matchedComponent != null) {
                        break;
                    }
                }
            } catch (NoSuchInterfaceException e1) {
                // do nothing
                matchedComponent = null;
            }

            try {
                String componentName = Fractal.getNameController(component).getFcName();
                if (componentName.equals(name)) {
                    matchedComponent = component;
                    break;
                }
            } catch (NoSuchInterfaceException e) {
                // do nothing, return null
                matchedComponent = null;
            }
        }
        return matchedComponent;
    }

    public static Component getComponentByName(final ContentController parentContentController,
            final String name) {
        // List content controller subcomponents
        for (Component component : parentContentController.getFcSubComponents()) {

            try {
                String componentName = Fractal.getNameController(component).getFcName();
                if (componentName.equals(name)) {
                    return component;
                }
            } catch (NoSuchInterfaceException e) {
                // do nothing
            }
        }

        return null;
    }

    public static List<Component> getComponentListByPrefix(
            final ContentController parentContentController, final String prefix) {
        List<Component> components = new ArrayList<Component>();
        // List content controller subcomponents
        for (Component component : parentContentController.getFcSubComponents()) {

            try {
                String componentName = Fractal.getNameController(component).getFcName();
                if (componentName.startsWith(prefix)) {
                    components.add(component);
                }
            } catch (NoSuchInterfaceException e) {
                // do nothing
            }
        }

        return components;
    }

    /**
     * A utility function to start the given fractal component.
     * 
     * @param component
     *            the fractal component
     * 
     * @return true if the component was found and stopped, false otherwise
     * @throws NoSuchInterfaceException
     *             : impossible to stop the component
     * @throws IllegalLifeCycleException
     *             : impossible to stop the component
     */
    public static final boolean startComponent(final Component component)
            throws NoSuchInterfaceException, IllegalLifeCycleException {
        boolean result = false;

        LifeCycleController lifeCycleController = Fractal.getLifeCycleController(component);

        if (lifeCycleController != null
                && LifeCycleController.STOPPED.equals(lifeCycleController.getFcState())) {
            lifeCycleController.startFc();
            result = true;
        }

        return result;
    }

    /**
     * A utility function to stop the given fractal component.
     * 
     * @param component
     *            the fractal component to stop
     * 
     * @return true if the component was found and stopped, false otherwise
     * @throws NoSuchInterfaceException
     *             : impossible to stop the component
     * @throws IllegalLifeCycleException
     *             : impossible to stop the component
     */
    public static final boolean stopComponent(final Component component)
            throws NoSuchInterfaceException, IllegalLifeCycleException {

        boolean result = false;
        LifeCycleController lifeCycleController = Fractal.getLifeCycleController(component);
        if (LifeCycleController.STARTED.equals(lifeCycleController.getFcState())) {
            lifeCycleController.stopFc();
            result = true;
        }
        return result;
    }

    /**
     * A utility function to stop the given fractal composite. TODO: Must be
     * remove when the bug no JBOSS environment would be resolved.
     * 
     * @param composite
     * @throws NoSuchInterfaceException
     * @throws IllegalLifeCycleException
     * @throws NoSuchInterfaceException
     */
    public static final void stopComposite(final Component composite)
            throws IllegalLifeCycleException, NoSuchInterfaceException {
        // the subcomponent content controller
        ContentController subContentController = null;
        ContentController parentContentController = Fractal.getContentController(composite);

        // List content controller sub components
        for (int i = parentContentController.getFcSubComponents().length; i != 0; i--) {
            Component component = parentContentController.getFcSubComponents()[i - 1];
            // if the component is a composite, stop the composite
            // recursively
            try {
                subContentController = Fractal.getContentController(component);
                if (subContentController.getFcSubComponents().length > 0) {
                    stopComposite(component);
                }
            } catch (NoSuchInterfaceException e) {
                // stop the component
                LifeCycleController lifeCycleController = Fractal.getLifeCycleController(component);
                if (LifeCycleController.STARTED.equals(lifeCycleController.getFcState())) {
                    lifeCycleController.stopFc();
                }
            }
        }
    }

    /**
     * Add a component in a composite component.
     * 
     * @param newComponent
     *            The new component
     * @param parentComponent
     *            The parent component
     * @param listOfBindings
     *            the list of binding to create between this new component and
     *            the others
     * @throws NoSuchInterfaceException
     *             : Impossible to add a component
     * @throws IllegalLifeCycleException
     *             : Impossible to add a component
     * @throws IllegalContentException
     *             : Impossible to add a component
     * @throws IllegalBindingException
     *             : Impossible to add a component
     */
    public static final void addComponent(final Component newComponent,
            final Component parentComponent, final List<Binding> listOfBindings)
            throws NoSuchInterfaceException, IllegalContentException, IllegalLifeCycleException,
            IllegalBindingException {
        // Get the parent controller
        ContentController parentContentController = Fractal.getContentController(parentComponent);

        parentContentController.addFcSubComponent(newComponent);

        // Get the binding controller of the new component
        BindingController cBindingController = Fractal.getBindingController(newComponent);

        // Add all the bindings
        if (listOfBindings != null) {
            for (int i = 0; i < listOfBindings.size(); i++) {
                cBindingController.bindFc((listOfBindings.get(i)).getClientInterfaceName(),
                        (listOfBindings.get(i)).getServerInterface());
            }
        }
    }

    /**
     * A utility function allowing to get a component LifeCycleController from a
     * content controller.
     * 
     * @param parentContentController
     *            the parent content controller
     * @param name
     *            the name of the component
     * @return the LifeCycleController of the component, null if not found
     */
    public static final LifeCycleController getLifeCycleControllerByName(
            final ContentController parentContentController, final String name) {

        LifeCycleController lifeCycleController = null;

        Component comp = getRecursiveComponentByName(parentContentController, name);

        if (comp != null) {
            try {
                lifeCycleController = Fractal.getLifeCycleController(comp);
            } catch (NoSuchInterfaceException e) {
                // do nothing, return null
                lifeCycleController = null;
            }
        }
        return lifeCycleController;
    }
}
