/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.service;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.petals.kernel.api.service.Location;
import org.ow2.petals.kernel.api.service.ServiceEndpoint;
import org.w3c.dom.Document;

/**
 * The PEtALS ESB service endpoint implementation
 * 
 * @author chamerling - eBM WebSourcing
 * 
 */
public class ServiceEndpointImpl implements ServiceEndpoint, Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = 3846565220895975538L;

    private Document description;

    private String endpointName;

    private QName serviceName;

    private Location location;

    private List<QName> interfacesName;
    
    private EndpointType type;

    /**
     * 
     */
    public ServiceEndpointImpl() {
        this(null, null, new ArrayList<QName>(0), null, new Location());
    }

    /**
     * 
     * @param endpointName
     * @param serviceName
     * @param interfacesName
     * @param description
     * @param location
     */
    public ServiceEndpointImpl(String endpointName, QName serviceName, List<QName> interfacesName,
            Document description, Location location) {
        super();
        this.endpointName = endpointName;
        this.serviceName = serviceName;
        this.interfacesName = interfacesName;
        this.description = description;
        this.location = location;
    }

    /**
     * {@inheritDoc}
     */
    public Document getDescription() {
        return this.description;
    }

    /**
     * {@inheritDoc}
     */
    public String getEndpointName() {
        return this.endpointName;
    }

    /**
     * {@inheritDoc}
     */
    public List<QName> getInterfacesName() {
        return this.interfacesName;
    }

    /**
     * {@inheritDoc}
     */
    public Location getLocation() {
        return this.location;
    }

    /**
     * {@inheritDoc}
     */
    public QName getServiceName() {
        return this.serviceName;
    }

    public void setDescription(Document description) {
        this.description = description;
    }

    public void setEndpointName(String endpointName) {
        this.endpointName = endpointName;
    }

    public void setServiceName(QName serviceName) {
        this.serviceName = serviceName;
    }

    public void setLocation(Location location) {
        this.location = location;
    }

    public void setInterfacesName(List<QName> interfacesName) {
        this.interfacesName = interfacesName;
    }

    @Override
    public String toString() {
        return this.serviceName + " ->" + this.endpointName + " (" + this.getType() + "):"
                + this.location.getSubdomainName() + "/" + this.location.getContainerName() + "/"
                + this.location.getComponentName();
    }

    public EndpointType getType() {
        return this.type;
    }

    public void setType(EndpointType type) {
        this.type = type;
    }

}
