/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 OW2,
 * http://www.ow2.org/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.transport.platform.nio.client;

import org.apache.commons.pool.BaseKeyedPoolableObjectFactory;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.topology.TopologyService;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.transport.TransportException;
import org.ow2.petals.transport.platform.nio.selector.NioSelector;
import org.ow2.petals.util.LoggingUtil;

/**
 * NIO Client factory.
 * 
 * @author Christophe DENEUX (Capgemini Sud)
 *
 */
public class NioClientAgentPoolObjectFactory extends BaseKeyedPoolableObjectFactory {

    private final TopologyService topologyService;

    private final NioSelector selectorAgent;

    private final Logger logger;

    private final LoggingUtil log;

    private final long connectionTimeout;

    public NioClientAgentPoolObjectFactory(final TopologyService topologyService,
            final NioSelector selectorAgent, final long connectionTimeout, final Logger logger) {
        this.topologyService = topologyService;
        this.selectorAgent = selectorAgent;
        this.logger = logger;
        this.log = new LoggingUtil(logger);
        this.connectionTimeout = connectionTimeout;
    }

    @Override
    public Object makeObject(final Object key) throws Exception {

        final String containerName = (String)key;

        if (this.logger.isLoggable(BasicLevel.DEBUG)) {
            this.log.debug("Creating new connection to container '" + containerName + "'");
        }

        final ContainerConfiguration containerConfiguration;
        try {
            containerConfiguration = this.topologyService.getContainerConfiguration(containerName);
        } catch (PetalsException e) {
            throw new TransportException("Cannot get the information for the container '"
                    + containerName + "'", e);
        }

        if (containerConfiguration == null) {
            throw new TransportException("Cannot get the information for the container "
                    + containerName);
        }

        final String host = containerConfiguration.getHost();
        final int port = containerConfiguration.getTCPPort();
        final NioClientAgent clientConnection = new NioClientAgent(this.logger, this.selectorAgent);
        if (this.logger.isLoggable(BasicLevel.DEBUG)) {
            this.log.debug("Connecting to " + host + ":" + port + " (Connection timeout: "
                    + this.connectionTimeout + ")");
        }
        clientConnection.connect(host, port, this.connectionTimeout);

        if (this.logger.isLoggable(BasicLevel.DEBUG)) {
            log.debug("Connection established to " + host + ":" + port);
        }

        return clientConnection;
    }

    @Override
    public void destroyObject(final Object key, final Object obj) throws Exception {
        final NioClientAgent nioClientAgent = (NioClientAgent)obj;

        if (this.logger.isLoggable(BasicLevel.DEBUG)) {
            this.log.debug("The NIO client agent connected to the container '" + key
                    + "' is removed from the pool.");
        }
        nioClientAgent.disconnect();
    }

    @Override
    public boolean validateObject(Object key, Object obj) {
        final NioClientAgent nioClientAgent = (NioClientAgent)obj;

        return nioClientAgent.isUsable();
    }

}
