/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.ws;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import java.util.Set;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.kernel.configuration.DomainConfiguration;
import org.ow2.petals.kernel.configuration.JndiConfiguration;
import org.ow2.petals.kernel.configuration.SubDomainConfiguration;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.TopologyService;
import org.ow2.petals.kernel.ws.api.to.ContainerInformation;
import org.ow2.petals.kernel.ws.api.to.ContainerInformation.NodeType;
import org.ow2.petals.kernel.ws.api.to.DomainInformation;
import org.ow2.petals.kernel.ws.api.to.JmxServiceInformation;
import org.ow2.petals.kernel.ws.api.to.JndiInformation;
import org.ow2.petals.kernel.ws.api.to.RegistryServiceInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation.SubdomainMode;
import org.ow2.petals.kernel.ws.api.to.TransportServiceInformation;
import org.ow2.petals.kernel.ws.api.to.WebServiceServiceInformation;
import org.ow2.petals.tools.ws.KernelWebService;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.StringHelper;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "service", signature = KernelWebService.class),
        @Interface(name = "webservice", signature = TopologyService.class) })
public class StandaloneTopologyServiceImpl implements TopologyService, KernelWebService {

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    @org.objectweb.fractal.fraclet.annotation.annotations.Service(name = "component")
    private Component component;

    @Requires(name = "configuration", signature = ConfigurationService.class)
    private ConfigurationService configurationService;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.debug("Starting...");
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.debug("Stopping...");
    }

    /**
     * @return the component
     */
    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     */
    public JndiInformation getJndiInformation() throws PEtALSWebServiceException {
        JndiInformation result = null;

        DomainConfiguration domainConfiguration = this.configurationService.getDomainConfiguration();

        if (domainConfiguration == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the domain configuration.");
            throw new PEtALSWebServiceException(errorSB.toString());
        }
        JndiConfiguration jndiConfiguration = domainConfiguration.getJndiConfiguration();

        if (jndiConfiguration == null) {
            return result;
        }

        if (StringHelper.isNullOrEmpty(jndiConfiguration.getJndiFactory())) {
            return result;
        }

        if ((jndiConfiguration.getJndiProviderUrl() == null) 
                || (StringHelper.isNullOrEmpty(jndiConfiguration.getJndiProviderUrl().toASCIIString()))) {
            return result;
        }

        result = new JndiInformation();

        result.setProviderUrl(jndiConfiguration.getJndiProviderUrl().toString());
        result.setFactory(jndiConfiguration.getJndiFactory());

        if (! StringHelper.isNullOrEmpty(jndiConfiguration.getJndiSecurityPrincipal())) {
            result.setSecurityPrincipal(jndiConfiguration.getJndiSecurityPrincipal());
        }

        if (! StringHelper.isNullOrEmpty(jndiConfiguration.getJndiSecurityCredentials())) {
            result.setSecurityPrincipal(jndiConfiguration.getJndiSecurityCredentials());
        }
        
        result.setBatchSize(domainConfiguration.getJndiConfiguration().getJndiBatchSize());
        result.setPoolSize(domainConfiguration.getJndiConfiguration().getJndiPoolSize());
        
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public void addContainerInformation(ContainerInformation addedContainer, String subdomainName) throws PEtALSWebServiceException {

        StringBuilder errorSB = new StringBuilder();
        errorSB.append("Can't add a container to a standalone topology");

        throw new PEtALSWebServiceException(errorSB.toString());
    }

    public void removeContainerInformation(String containerName) throws PEtALSWebServiceException {
        StringBuilder errorSB = new StringBuilder();
        errorSB.append("Can't remove a container from a standalone topology");

        throw new PEtALSWebServiceException(errorSB.toString());
    }

    /**
     * {@inheritDoc}
     */
    public void addSubdomain(String name, String description, SubdomainMode subdomainMode, boolean networkTimeSynchronized) throws PEtALSWebServiceException {
        StringBuilder errorSB = new StringBuilder();
        errorSB.append("Can't add a subdomain to a standalone topology");

        throw new PEtALSWebServiceException(errorSB.toString());
    }

    public void removeSubdomain(String subdomainName) throws PEtALSWebServiceException {
        StringBuilder errorSB = new StringBuilder();
        errorSB.append("Can't remove a subdomain from a standalone topology");

        throw new PEtALSWebServiceException(errorSB.toString());
    }

    /**
     * {@inheritDoc}
     */
    public void addSubdomainWithContainers(SubdomainInformation subdomain) throws PEtALSWebServiceException {
        StringBuilder errorSB = new StringBuilder();
        errorSB.append("Can't add a subdomain to a standalone topology");

        throw new PEtALSWebServiceException(errorSB.toString());
    }

    /**
     * {@inheritDoc}
     */
    public List<ContainerInformation> getAllContainerInformation() throws PEtALSWebServiceException {
        List<ContainerInformation> result = new ArrayList<ContainerInformation>();
        ContainerConfiguration cc = this.configurationService.getContainerConfiguration();
        ContainerInformation ci = this.getContainerInformation(cc);
        result.add(ci);
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public ContainerInformation getContainerInformation(String containerId)
            throws PEtALSWebServiceException {
        ContainerInformation result = null;
        if (!this.configurationService.getContainerConfiguration().getName().equals(containerId)) {
            throw new PEtALSWebServiceException("This is not a valid container name (mine is "
                    + this.configurationService.getContainerConfiguration().getName() + ")");
        }

        result = this
                .getContainerInformation(this.configurationService.getContainerConfiguration());

        return result;
    }

    /**
     * @param containerId
     * @param containerConfiguration
     * @return
     */
    private ContainerInformation getContainerInformation(
            ContainerConfiguration containerConfiguration) throws PEtALSWebServiceException {
        ContainerInformation result = new ContainerInformation();

        //Sets the "generic" part of the container information
        result.setName(containerConfiguration.getName());
        result.setHost(containerConfiguration.getHost());
        result.setDescription(containerConfiguration.getDescription());
        result.setUser(containerConfiguration.getUser());
        result.setPassword(containerConfiguration.getPassword());

        //Configure the registry mode
        NodeType nodeType = null;
        switch ( containerConfiguration.getRegistryMode()) {
            case MASTER :
                nodeType = NodeType.MASTER;
                break;
            case SLAVE :
                nodeType = NodeType.SLAVE;
                break;
            case PEER :
                nodeType = NodeType.PEER;
                break;
            case STANDALONE :
                nodeType = NodeType.STANDALONE;
                break;
            default :
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Unsupported registry mode");
                throw new PEtALSWebServiceException(errorSB.toString());
        }
        result.setNodeType(nodeType);

        //Configure the Webservice part of the container information
        WebServiceServiceInformation wsInfo = new WebServiceServiceInformation();
        wsInfo.setPort(containerConfiguration.getWebservicePort());
        wsInfo.setPrefix(containerConfiguration.getWebservicePrefix());
        result.setWebserviceService(wsInfo);

        //Configure the registry part of the container information
        RegistryServiceInformation registryInfo = new RegistryServiceInformation();
        registryInfo.setPort(containerConfiguration.getRegistryPort());
        result.setRegistryService(registryInfo);

        //Configure the transport part of the container information
        TransportServiceInformation transportServiceInfo = new TransportServiceInformation();
        transportServiceInfo.setTcpPort(containerConfiguration.getTCPPort());
        result.setTransportService(transportServiceInfo);

        //Configure the jmx part of the container information
        JmxServiceInformation jmxInfo = new JmxServiceInformation();
        jmxInfo.setRmiPort(containerConfiguration.getJmxRMIConnectorPort());
        result.setJmxService(jmxInfo);

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public SubdomainInformation getSubdomainByName(String subdomainName) throws PEtALSWebServiceException {

        if (subdomainName == null) {
            throw new PEtALSWebServiceException("No subdomain name found");
        }

        SubDomainConfiguration localSubdomainConfiguration = this.getLocalContainerSubdomainConfiguration();

        String localSubdomainName = localSubdomainConfiguration.getName();
        if (! localSubdomainName.equals(subdomainName)) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("The subdomain " + subdomainName + " can't be retrieved from the local topology.");
            errorSB.append("This container runs in standalone mode, and the only subdomain that can be ");
            errorSB.append("retrieved is the local subdomain, which name is : " + localSubdomainName);
            throw new PEtALSWebServiceException(errorSB.toString());
        }
        
        return this.transformSubdomainConfigurationToSubdomainInformation(localSubdomainConfiguration);
    }

    private SubDomainConfiguration getLocalContainerSubdomainConfiguration() throws PEtALSWebServiceException {
        SubDomainConfiguration subdomainConfiguration = null;

        if ((subdomainConfiguration = this.getLocalContainerSubdomainConfiguration()) == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the subdomain containing the local container");
        }

        return subdomainConfiguration;
    }

    private SubdomainInformation transformSubdomainConfigurationToSubdomainInformation(
            SubDomainConfiguration subdomainConfiguration) throws PEtALSWebServiceException {
        SubdomainInformation result = null;

        if (subdomainConfiguration == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("No subdomain configuration found");
            throw new PEtALSWebServiceException(errorSB.toString());
        }

        result = new SubdomainInformation();

        result.setNetworkTimeSynchronized(subdomainConfiguration.isNetworkTimeSynchronized());
        result.setDescription(subdomainConfiguration.getDescription());
        result.setName(subdomainConfiguration.getName());

        switch (subdomainConfiguration.getMode()) {
            case MASTER_SLAVE :
                result.setSubdomainMode(SubdomainInformation.SubdomainMode.MASTERSLAVE);
                break;
            case FLOODING :
                result.setSubdomainMode(SubdomainInformation.SubdomainMode.FLOODING);
                break;
            case STANDALONE :
                result.setSubdomainMode(SubdomainInformation.SubdomainMode.STANDALONE);
                break;
            default :
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Unrecognized domain sudomain mode");
                throw new PEtALSWebServiceException(errorSB.toString());
        }

        //Start to transform local containers configuration to containers information
        Set<ContainerConfiguration> registeredContainersConfigurations =
                this.configurationService.getContainersConfiguration();

        List<ContainerInformation> registeredContainersInformations =
                new ArrayList<ContainerInformation>();

        if (registeredContainersConfigurations.size() != 1) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Too many containers registered for a standalone topology");
            throw new PEtALSWebServiceException(errorSB.toString());
        }

        for (ContainerConfiguration containerConfiguration : registeredContainersConfigurations) {
            if (! containerConfiguration.getName().equals(this.configurationService.getContainerConfiguration().getName())) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Registerd container name does not match local container name");
                throw new PEtALSWebServiceException(errorSB.toString());
            }

            registeredContainersInformations.add(this.getContainerInformation(containerConfiguration));
        }

        result.setContainers(registeredContainersInformations);

        return result;
    }

    public DomainInformation getDomainInformation() throws PEtALSWebServiceException {
        DomainInformation domainInformation = new DomainInformation();

        try {
            domainInformation.setName(this.configurationService.getDomainConfiguration().getName());
            domainInformation.setDescription(this.configurationService.getDomainConfiguration().getDescription());
            domainInformation.setJndiInformation(this.getJndiInformation());
            domainInformation.setMode(this.domainModeTransformer(this.configurationService.getDomainConfiguration().getMode()));

            List<SubdomainInformation> subdomainsInformations = new ArrayList<SubdomainInformation>();

            for (SubDomainConfiguration subdomainConfiguration : this.configurationService.getSubDomainsConfiguration()) {
                SubdomainInformation subInfo = this.transformSubdomainConfigurationToSubdomainInformation(subdomainConfiguration);
                subdomainsInformations.add(subInfo);
            }

            domainInformation.setSubdomains(subdomainsInformations);

        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }

        return domainInformation;
    }

    private org.ow2.petals.kernel.ws.api.to.DomainInformation.DomainMode domainModeTransformer(
            org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode domainMode)
            throws PetalsException {

        switch (domainMode) {
            case STATIC :
                return org.ow2.petals.kernel.ws.api.to.DomainInformation.DomainMode.STATIC;
            case STANDALONE :
                return org.ow2.petals.kernel.ws.api.to.DomainInformation.DomainMode.STANDALONE;
            default :
                throw new PetalsException("Invalid domain mode");
        }
    }
}
